class BaselineResource < ActiveRecord::Base
  unloadable

  belongs_to :baseline
  has_many :properties,
    :class_name => "BaselineProperty",
    :dependent => :delete_all,
    :order => 'name ASC'

  accepts_nested_attributes_for :properties

  def self.create_from_resource_status(rs)
    unless rs.is_a? ResourceStatus
      raise "Expected a ResourceStatus, but got #{rs.inspect}"
    end

    create(:resource_type         => rs.resource_type,
           :title                 => rs.title,
           :properties_attributes => rs.events.map {|ev|
             { :name => ev.property, :value => ev.previous_value }
           })
  end

  def self.new_from_resource_status(rs)
    unless rs.is_a? ResourceStatus
      raise "Expected a ResourceStatus, but got #{rs.inspect}"
    end

    new(:resource_type         => rs.resource_type,
        :title                 => rs.title,
        :properties_attributes => rs.events.map {|ev|
          { :name => ev.property, :value => ev.previous_value }
        })
  end

  def resource_name
    return "#{resource_type}[#{title}]"
  end
  alias :name :resource_name

  def validate
    super

    if baseline and baseline.resources.find_by_resource_type_and_title(resource_type, title)
      errors.add_to_base("Resource #{resource_name} already exists in this baseline")
    end
  end

  # Replace this baseline resource with one from a real report.
  def replace_with(resource)
    baseline = self.baseline
    baseline.transaction do
      # Remove this from the baseline, and also the database, since it is
      # being replaced wholesale.  If we unify these, we need to make the
      # self.destroy dependent on the "reference count" of this row being
      # zero; only destroy if nothing refers to it.  Maybe move that out of
      # the method into a nightly task, even.
      baseline.resources.delete(self)
      self.destroy

      # Create and store the new resource that replaces this one, and reload
      # the baseline object so that it reflects the change in the resource
      # association.
      baseline.resources << BaselineResource.create_from_resource_status(resource)
      baseline.save!
      baseline.reload
    end
  end
end
